const readline = require('readline');
const read = require('fs-readdir-recursive');
const fs = require('fs');

const reader = readline.createInterface({
  input: process.stdin,
  output: process.stdout,
});

console.log(
  '\nInnan vi forsätter med uppgraderingen måste du se' +
    '\ntill att du står i rotmappen av projektet du vill uppgradera.'
);
console.log('\nDessutom att du inte har några ändringar som inte är committade' + '\ni din branch.');

console.log(
  '\nObs! att detta script endast går igenom och uppdaterar htmlfilerna i ditt projekt,\n' +
    'finns det ändra typer av filer som innehåller kod från designsystem som t.ex. ts, js, less, scss etc. så får ni manuellt justera dessa'
);

reader.question('\nVill du fortsätta? (y/n)', async function (svar) {
  if (svar.toLowerCase() == 'y') {
    console.log('Letar igenom alla html filer.');
    const htmlFiles = read(process.cwd())
      .filter(item => item.endsWith('.html'))
      .filter(item => !item.startsWith('node_modules'))
      .filter(item => !item.startsWith('dist'));

    
    findAccordions(htmlFiles);
    findBifogaFil(htmlFiles);
    findDropdown(htmlFiles);
    findMultiSelect(htmlFiles);
    findMultitext(htmlFiles);
    findTabs(htmlFiles);
    findHeader(htmlFiles);
    findHorisontellMeny(htmlFiles);
    findKort(htmlFiles);
    
    await reader.close();
  } else {
    await reader.close();
  }
});

function findHeader(htmlFiles) {
  let findAny = false;
  
  for (const htmlFile of htmlFiles) {
    findAny = lookingHeader(htmlFile) || findAny;
  }
  if (findAny) {
    console.log('För varje sidhuvud komponent uppdatera html enligt nytt mönster, då både html strukturen och CSS klasserna har uppdaterats kolla: ');
    console.log('https://styleguide.jordbruksverket.se/komponenter/sidhuvud.html');
  } else {
    console.log('Hittade inga sidhuvud komponenter som behöver uppdateras');
  }
  console.log('\n');
}

const lookingHeader = function (htmlFile) {
  let data = fs.readFileSync(htmlFile, 'utf8');
  if (/class=".*c-header c-header--white*/i.test(data)) {
    
    console.log('OBS! Har hittat en eller flera sidhuvud komponent(er) '
    + 'som behöver uppdateras manuellt i filen ' + htmlFile);
    return true;
  }
  return false;
}

function findHorisontellMeny(htmlFiles) {
  let findAny = false;
  
  for (const htmlFile of htmlFiles) {
    findAny = lookingForHorisontellMeny(htmlFile) || findAny;
  }
  if (findAny) {
    console.log('För varje horisontell meny komponent uppdatera html enligt nytt mönster, då både html strukturen och CSS klasserna har uppdaterats kolla: ');
    console.log('https://styleguide.jordbruksverket.se/komponenter/meny/horisontell-meny.html');
  } else {
    console.log('Hittade inga horisontell meny komponenter som behöver uppdateras');
  }
  console.log('\n');
}

const lookingForHorisontellMeny = function (htmlFile) {
  let data = fs.readFileSync(htmlFile, 'utf8');
  if (/class=".*c-menu c-menu--white*/i.test(data)) {
    
    console.log('OBS! Har hittat en eller flera horisontell meny komponent(er) '
    + 'som behöver uppdateras manuellt i filen ' + htmlFile);
    return true;
  }
  return false;
}

function findKort(htmlFiles) {
  let findAny = false;
  
  for (const htmlFile of htmlFiles) {
    findAny = lookingForKort(htmlFile) || findAny;
  }
  if (findAny) {
    console.log('För varje jv-card komponent uppdatera html enligt nytt mönster, då både html strukturen och CSS klasserna har uppdaterats kolla: ');
    console.log('https://styleguide.jordbruksverket.se/komponenter/kort.html');
  } else {
    console.log('Hittade inga jv-card komponenter som behöver uppdateras');
  }
  console.log('\n');
}

const lookingForKort = function (htmlFile) {
  let data = fs.readFileSync(htmlFile, 'utf8');
  if (/<jv-card.*/i.test(data)) {
    
    const matchingArray = data.match(/<jv-card([^>]+)>$/gm);
    if (!matchingArray) {
      return false;
    }
    
    if (matchingArray.some(matchingString => matchingString.includes('img-ref'))) {
      console.log('OBS! Har hittat en eller flera jv-card komponent(er) '
        + 'som behöver uppdateras manuellt i filen ' + htmlFile);
      return true;
    }
  }
}

function findTabs(htmlFiles) {
  let findAnyTabs = false;
  
  for (const htmlFile of htmlFiles) {
    findAnyTabs = lookingForTabs(htmlFile) || findAnyTabs;
  }
  if (findAnyTabs) {
    console.log('För varje jv-tabs komponent behöver ni ändra värdet i "tabs-content-ref"'
     + ' och "tabs-ref" attributet så att de följer ett strikt json format ex:');
    console.log('tabs-content-ref=\'["flik1 content"]\' och tabs-ref=\'["flik1"]\'');
  } else {
    console.log('Hittade inga jv-tabs komponenter som behöver uppdateras');
  }
  console.log('\n');
}

const lookingForTabs = function (htmlFile) {
  let data = fs.readFileSync(htmlFile, 'utf8');
  if (/<jv-tabs.*/i.test(data)) {
    
    const matchingArray = data.match(/<jv-tabs([^>]+)>$/gm);
    if (!matchingArray) {
      return false;
    }
    
    if (matchingArray.some(matchingString => matchingString.includes('tabs-content-ref="['))) {
      console.log('OBS! Har hittat en eller flera jv-tabs komponent(er) '
      + 'som behöver uppdateras manuellt i filen ' + htmlFile);
     return true;
    } else if (matchingArray.some(matchingString => matchingString.includes('tabs-ref="['))) {
      console.log('OBS! Har hittat en eller flera jv-tabs komponent(er) '
      + 'som behöver uppdateras manuellt i filen ' + htmlFile);
     return true;
    }
  }
  return false;
}

function findMultitext(htmlFiles) {
  let findAnyMultitext = false;
  
  for (const htmlFile of htmlFiles) {
    findAnyMultitext = lookingForMultitext(htmlFile) || findAnyMultitext;
  }
  if (findAnyMultitext) {
    console.log('För varje multitext behöver ni ändra värdet i value attributet så att de följer ett strikt json format ex:');
    console.log('value=\'["Test"]\'');
  } else {
    console.log('Hittade inga multitext komponenter som behöver uppdateras');
  }
  console.log('\n');
}

const lookingForMultitext = function (htmlFile) {
  let data = fs.readFileSync(htmlFile, 'utf8');
  if (/<jv-multitext.*/i.test(data)) {
    
    const matchingArray = data.match(/<jv-multitext([^>]+)>$/gm);
    if (!matchingArray) {
      return false;
    }
    
    if (matchingArray.some(matchingString => matchingString.includes('value="['))) {
      console.log('OBS! Har hittat en eller flera jv-multitext komponent(er) '
      + 'som behöver uppdateras manuellt i filen ' + htmlFile);
     return true;
    }
  }
  return false;
}

function findMultiSelect(htmlFiles) {
  let findAnyMultiselect = false;
  
  for (const htmlFile of htmlFiles) {
    findAnyMultiselect = lookingForMultiSelect(htmlFile) || findAnyMultiselect;
  }
  if (findAnyMultiselect) {
    console.log('För varje multi-select behöver ni ändra värdet i options och value attributet så att de följer ett strikt json format ex:');
    console.log('options=\'["Norge", "Danmark"]\' eller value=\'["Norge"]\'');
  } else {
    console.log('Hittade inga multi-select komponenter som behöver uppdateras');
  }
  console.log('\n');
}

const lookingForMultiSelect = function (htmlFile) {
  let data = fs.readFileSync(htmlFile, 'utf8');
  if (/<jv-multi-select.*/i.test(data)) {
    
    const matchingArray = data.match(/<jv-multi-select([^>]+)>$/gm);
    if (!matchingArray) {
      return false;
    }
    
    if (matchingArray.some(matchingString => matchingString.includes('options="['))) {
      console.log('OBS! Har hittat en eller flera jv-multi-select komponent(er) '
       + 'som behöver uppdateras manuellt i filen ' + htmlFile);
      return true;
    } else if (matchingArray.some(matchingString => matchingString.includes('value="['))) {
      console.log('OBS! Har hittat en eller flera jv-multi-select komponent(er) '
      + 'som behöver uppdateras manuellt i filen ' + htmlFile);
     return true;
    }
  }
  return false;
}

function findDropdown(htmlFiles) {
  let findAnyDropdowns = false;
  
  for (const htmlFile of htmlFiles) {
    findAnyDropdowns = lookingForDropdown(htmlFile) || findAnyDropdowns;
  }
  if (findAnyDropdowns) {
    console.log('För varje dropdown behöver ni ändra värdet i options attributet att de följer ett strikt json format ex:');
    console.log('options=\'{"size": "large"}\'');
  } else {
    console.log('Hittade inga dropdown komponenter som behöver uppdateras');
  }
  console.log('\n');
}

const lookingForDropdown = function (htmlFile) {
  let data = fs.readFileSync(htmlFile, 'utf8');
  if (/<jv-dropdown.*/i.test(data)) {
    
    const matchingArray = data.match(/<jv-dropdown([^>]+)>$/gm);
    if (!matchingArray) {
      return false;
    }
    
    if (matchingArray.some(matchingString => matchingString.includes('options="{'))) {
      console.log('OBS! Har hittat en eller flera jv-dropdown komponent(er) '
       + 'som behöver uppdateras manuellt i filen ' + htmlFile);
      return true;
    }
    
  }
  return false;
}

function findBifogaFil(htmlFiles) {
  let findAnyBifogaFil = false;
  
  for (const htmlFile of htmlFiles) {
    findAnyBifogaFil = lookingForBifogaFil(htmlFile) || findAnyBifogaFil;
  }
  if (findAnyBifogaFil) {
    console.log('För varje Bifoga fil komponent behöver du lägga in ett <input id="unikt_id" type="file"/> som en slot');
  } else {
    console.log('Hittade inga Bifoga fil komponent');
  }
  console.log('\n');
}

const lookingForBifogaFil = function (htmlFile) {
  let data = fs.readFileSync(htmlFile, 'utf8');
  if (/<jv-bifoga-fil.*/i.test(data)) {
    console.log('OBS! Har hittat en eller flera jv-bifoga-fil komponent(er) '
     + 'som eventuellt måste uppdateras manuellt i filen ' + htmlFile);
    return true;
  }
  return false;
}

function findAccordions(htmlFiles) {
  let findAnyAccordionV1 = false;
  let findAnyAccordionJS = false;
  for (const htmlFile of htmlFiles) {
    findAnyAccordionV1 = lookingForAccordionV1(htmlFile) || findAnyAccordionV1;
    findAnyAccordionJS = lookingForAccordionJS(htmlFile) || findAnyAccordionJS;
  }
  if (findAnyAccordionV1 === true) {
    console.log(
      '** 1.1 Accordion : För varje gammal accordion behöver du ändra komponent-deklaration till jv-accordion-v1 och .c-accordion-group till .c-accordionv1-group.'
    );
  }
  if (findAnyAccordionJS === true) {
    console.log(
      '** 1.2 Accordion : Icke-webbkomponent accordions är nu helt-borta från sg-komponentbibliotek. Ersätt de manuellt med den nya accordion komponent.'
    );
  }
  console.log('\n');
}

const lookingForAccordionV1 = function (htmlFile) {
  let data = fs.readFileSync(htmlFile, 'utf8');
  if (/<jv-accordion.*>/i.test(data)) {
    console.log('** 1.1 Har hittat gammal accordions ' + htmlFile);
    return true;
  }
  return false;
};

const lookingForAccordionJS = function (htmlFile) {
  let data = fs.readFileSync(htmlFile, 'utf8');
  if (/rel="c-accordion-js"/i.test(data)) {
    console.log('** 1.2 Har hittat icke-webbkomponent accordions ' + htmlFile);
    return true;
  }
  return false;
};


reader.on('close', function () {
  console.log('\nAvslutar!');
  process.exit(0);
});

